# community.sap_install Ansible Collection

![Ansible Lint](https://github.com/sap-linuxlab/community.sap_install/actions/workflows/ansible-lint.yml/badge.svg?branch=main)

This Ansible Collection executes various SAP Software installations and configuration tasks for running SAP software on Linux operating systems; with handlers for SAP HANA database lifecycle manager (HDBLCM) and SAP Software Provisioning Manager (SWPM) for programmatic deployment of any SAP solution scenario.

This can be combined with other Ansible Collections to provide end-to-end automation, from download of SAP software installation media through to technical configuration and burstable SAP NetWeaver application servers (start/stop).

## Functionality

This Ansible Collection contains roles for:
  - **Preparing RHEL 7.6 and later, RHEL 8, and RHEL 9 systems for the installation of SAP HANA and NetWeaver software**
  - **Installing SAP HANA standalone and scale-out, in a non clustered and clustered environment**
  - **Installing SAP software via the SAP Software Provisioning Manager (SWPM)**

  - *NOTE*: This version of the collection redhat.sap_install contains the role `sap_swpm` with a support scope of *Technology Preview*
    for the following deployment scenarios:
    S/4HANA 2021 and S/4HANA 2022 single-host installation on x86_64: RHEL 8.4, 8.6, 9.0 and ppc64le: RHEL 8.4 and 8.6.

    Only the `default` mode of the role `sap_swpm` is supported.

    All other functionality of this role is currently not supported by Red Hat.

## Contents

An Ansible Playbook can call either an Ansible Role, or the individual Ansible Modules:

- **Ansible Roles** (runs multiple Ansible Modules)
- **Ansible Modules** (and adjoining Python/Bash Functions)

Within this Ansible Collection, there are various Ansible Roles and no custom Ansible Modules.

### Ansible Roles

| Name                                                                                                                                           | Summary                                                                | Support |
| :--- | :--- | :--- |
| [sap_general_preconfigure](https://github.com/sap-linuxlab/community.sap_install/tree/main/roles/sap_general_preconfigure)      	         | configure general OS settings for SAP software                         | Fully Supported |
| [sap_ha_install_hana_hsr](https://github.com/sap-linuxlab/community.sap_install/tree/main/roles/sap_ha_install_hana_hsr)        	         | install SAP HANA System Replication                                    | Technology Preview |
| [sap_ha_pacemaker_cluster](https://github.com/sap-linuxlab/community.sap_install/tree/main/roles/sap_ha_pacemaker_cluster)                     | install and configure pacemaker and SAP resources                      | Technology Preview |
| [sap_hana_install](https://github.com/sap-linuxlab/community.sap_install/tree/main/roles/sap_hana_install)                      	         | install SAP HANA via HDBLCM                                            | Fully Supported |
| [sap_hana_preconfigure](https://github.com/sap-linuxlab/community.sap_install/tree/main/roles/sap_hana_preconfigure)            	         | configure settings for SAP HANA database server                        | Fully Supported |
| [sap_netweaver_preconfigure](https://github.com/sap-linuxlab/community.sap_install/tree/main/roles/sap_netweaver_preconfigure)  	         | configure settings for SAP NetWeaver application server                | Fully Supported |
| [sap_swpm](https://github.com/sap-linuxlab/community.sap_install/tree/main/roles/sap_swpm)                                                     | install SAP Software via SWPM                                          | Technology Preview (for certain functionality only) |

**_Notes_**:

In general the "preconfigure" and "prepare" roles are prerequisites for the corresponding installation roles.
The logic has been separated to support a flexible execution of the different steps.

**_Notes:_**

- Ansible Playbook localhost executions may have limitations on SAP Software installations
- Ansible Roles for HA/DR are all designed for execution with Terraform

## Execution examples

There are various methods to execute the Ansible Collection, dependent on the use case. For more information, see the summary below:

| Execution Scenario                                                                                                                                                       | Use Case                                                                                     | Target              |
| ------------------------------------------------------------------------------------------------------------------------------------------------------------------------ | -------------------------------------------------------------------------------------------- | ------------------- |
| Ansible Playbook <br/>-> source Ansible Collection <br/>-> execute Ansible Task <br/>--> run Ansible Role <br/>---> run Ansible Module for Shell (built-in)<br/>---> ... | Complex executions with various interlinked activities;<br/> run in parallel or sequentially | Localhost or Remote |

## Requirements, Dependencies and Testing

### Operating System requirements

Supported for control nodes running collection redhat.sap_install or package rhel-system-roles-sap:
- Red Hat Enterprise Linux for SAP Solutions subscription or Red Hat Ansible Automation Platform subscription

Supported for managed nodes with the scope described above for RHEL 7.6 and later RHEL 7, RHEL 8, and RHEL 9 operating systems

## License

- [Apache 2.0](./LICENSE)

## Contributors

Contributors to the Ansible Roles within this Ansible Collection are shown in the README.md file of each role.
